<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus02600 extends \Zeedhi\Framework\Controller\Report  { 

    protected $novoCodigo;
    protected $consplan;
    protected $dataSourceManager;
    protected $codigo;
    protected $environment;
    protected $connection;
    protected $birtStrategy;

    public function __construct($crudService, $parameterBag, $consplan, $novoCodigo, $entityManager, $dataSourceManager, $environment, $connection, $reportService, $birtStrategy, $util) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->consplan = $consplan;
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->reportService = $reportService;
        $this->birtStrategy = $birtStrategy;
        $this->util = $util;
    }

    protected $reportMapping = array(
        'CUS02600' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'CDFILIAL'        => 'P_CDFILIAL',
                'CDSERVFILI'      => 'P_CDSERVFILI',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTFINAL'         => 'P_DTFINAL',
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTOPLAN',
                'TPDTCUS'         => 'P_TPDTCUS',
                'PRODESTOC'       => 'P_IDESTOCAVEIS',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'DTMESINIC'       => 'P_DTMESINIC',
                'DTMESFINA'       => 'P_DTMESFINA',
                'PRODNAOESTOC'    => 'P_IDNESTOCAVEIS',
                'CDPRODINICIO'    => 'P_CDPRODINI',
                'CDPRODFINAL'     => 'P_CDPRODFINAL',
                'CDARVPRODFINAL'  => 'P_CDARVPRODFINAL',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'NMTIPOCUSTOREAL' => 'P_NMTIPOCUSTOREAL',
                'NMTIPOCUSTOPLAN' => 'P_NMTIPOCUSTOPLAN',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'NRORG'           => 'P_NRORG'

            )
        ),
        'CUS02601' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTO',
                'TPDTCUS'         => 'P_IDDATAREF',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'DTMESINIC'       => 'P_DTMESCUSTPDR',
                'CDFILIAL'        => 'P_CDFILIAL',
                'CDSERVFILI'      => 'P_CDSERVFILI',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTFINAL'         => 'P_DTFINAL',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'CDPRODINICIO'    => 'P_CDPRODINIC',
                'CDPRODFINAL'     => 'P_CDPRODFINA',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'CDPICTPROD'      => 'P_MASK',
                'NRORG'           => 'P_NRORG'
            )
        ),
        'CUS02602' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'CDTIPOCUSTOPLAN' => 'P_NRTIPOCUSTO',
                'TPDTCUS'         => 'P_IDDATAREF',
                'CDTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'DTMESINIC'       => 'P_DTMESCUSTPDR',
                'CDFILIAL'        => 'P_CDFILIAL',
                'CDSERVFILI'      => 'P_CDSERVFILI',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTFINAL'         => 'P_DTFINAL',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'CDPRODINICIO'    => 'P_CDPRODINIC',
                'CDPRODFINAL'     => 'P_CDPRODFINA',
                'IDDIFERENCA'     => 'P_IDDIFERENCA',
                'CDPICTPROD'      => 'P_MASK',
                'NRORG'           => 'P_NRORG',
                'NMTIPOCUSTOPLAN'     => 'P_NMTIPOCUSTOPLAN',
                'NMTIPOCUSTOREAL'     => 'P_NMTIPOCUSTOREAL'
            )
        )
    );

    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->procedureConsumoPlanejado($row);
        $row["NRORG"] = $this->environment->getNrOrgTrab();
    }

    public function procedureConsumoPlanejado($row) {
        $dataFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dataInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);

        $tamanho = 20;
        $completaCom = 'z';
        $row['CDPRODFINAL'] = str_pad($row['CDPRODFINAL'], $tamanho, $completaCom, STR_PAD_RIGHT);
        $row['CDARVPRODFINAL'] = str_pad($row['CDARVPRODFINAL'], $tamanho, $completaCom, STR_PAD_RIGHT);

        if ($row['LISTASERV'] == 'REQUEST' || $row['TodosServicos'] == 'S') {
            $this->dataSourceName = 'servicos_por_grupo';
            $this->parameterBagColumns = array('CDFILIAL', 'CDOPERADOR', 'CDGRUPSERV', 'NRORG');

            $filterCriteria = new FilterCriteria($this->dataSourceName);
            
            $filterCriteria->addCondition('CDFILIAL',   $row['CDFILIAL']);
            $filterCriteria->addCondition('CDOPERADOR', $row['CDOPERADOR']);
            $filterCriteria->addCondition('CDGRUPSERV', $row['LISTAGRUPSERV']);
            $filterCriteria->addCondition('NRORG', $row['NRORG']);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            $row['CDSERVFILI'] = $this->retornaArrayServfili($dataSet->getRows());
            $row['CDSERVFILI'] = $this->util->insertLista($row['CDSERVFILI']);
            $row['LISTASERV'] = $this->retornaServicosConcatenados($dataSet->getRows());
        }else if($row['TodosServicos'] != 'S'){
            $row['CDSERVFILI'] = $this->util->insertLista(explode('_', $row['LISTASERV']));
        }

        $this->codigo = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);

        $connection = $this->entityManager->getConnection();
        
        $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $row['LISTASERV'],                   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],                   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],                     Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",                               Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "N",                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                                   Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $this->codigo,                       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $row['CDARVPRODINICIO'],                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $row['CDARVPRODFINAL'],                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dataInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dataFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();
        $this->params = array();
        $nrConsPlan = $this->codigo;
      
        $row['NRCONSPLAN']      = $nrConsPlan;
        $row['NRLISTA']         = $nrConsPlan;
        $row['DTPERIINIC']      = $row['DTINICIAL'];
        $row['DTPERIFINA']      = $row['DTFINAL'];
        $row['CDTIPOCUSTOPLAN'] =  (int)$row['CDTIPOCUSTOPLAN'];
        $row['CDTIPOCUSTOREAL'] =  (int)$row['CDTIPOCUSTOREAL'];

        $this->nrorg      = $row['NRORG'];
        $this->cdfilial   = $row['CDFILIAL'];
        $this->nrConsPlan = $nrConsPlan;

        // if ($row['CD_REPORT_NAME'] == 'Cus02600') {
        //     $porData = $this->curvaPorData($row);
        //     return $porData;

        // } else {
        //     $porProduto = $this->curvaPorProduto($row);
        //     return $porProduto;
        // }
    }

    public function curvaPorData ($row) {
        $this->dataSourceName = 'cus02600_abcplanrealdata';
        $this->parameterBagColumns = array('P_CDFILIAL', 'P_NRTIPOCUSTO', 'P_TIPODATA', 'P_NRTIPOCUSTOREAL', 'P_DTINICIAL', 'P_DTMESINIC', 'P_DTFINAL', 'P_DTMESFINA', 'P_NRCONSPLAN', 'P_CDPRODINIC', 'P_CDPRODFINA', 'P_IDESTOCAVEIS', 'P_IDNESTOCAVEIS', 'P_DIFERENCA', 'P_SERVICOS', 'P_GRUPSERV');

        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('P_CDFILIAL',     $row['CDFILIAL']);        
        $filterCriteria->addCondition('P_NRTIPOCUSTO',  $row['CDTIPOCUSTOPLAN']);
        $filterCriteria->addCondition('P_TIPODATA',     $row['TPDTCUS']);
        $filterCriteria->addCondition('P_NRTIPOCUSTOREAL', $row['CDTIPOCUSTOREAL']);
        $filterCriteria->addCondition('P_DTINICIAL',    $row['DTINICIAL']);
        $filterCriteria->addCondition('P_DTMESINIC',    $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTFINAL',      $row['DTFINAL']);
        $filterCriteria->addCondition('P_DTMESFINA',    $row['DTMESFINA']);
        $filterCriteria->addCondition('P_NRCONSPLAN',   $row['NRCONSPLAN']);
        $filterCriteria->addCondition('P_CDPRODINIC',   $row['CDPRODINICIO']);
        $filterCriteria->addCondition('P_CDPRODFINA',   $row['CDPRODFINAL']);
        $filterCriteria->addCondition('P_IDESTOCAVEIS', $row['PRODESTOC']);
        $filterCriteria->addCondition('P_IDNESTOCAVEIS', $row['PRODNAOESTOC']);
        $filterCriteria->addCondition('P_DIFERENCA',    $row['IDDIFERENCA']);
        $filterCriteria->addCondition('P_SERVICOS',     $row['LISTASERV']);
        $filterCriteria->addCondition('P_GRUPSERV',     $row['LISTAGRUPSERV']);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }

    public function curvaPorProduto ($row) {
        if ($row['CD_REPORT_NAME'] == "Cus02601") {
            $row['TIPO_REL'] = 'Q';
        } else if ($row['CD_REPORT_NAME']  == "Cus02602") {
            $row['TIPO_REL'] = 'C';
        } else {
            $row['TIPO_REL'] = '';
        }

        $this->dataSourceName = 'cus02600_abcplanrealproduto';
        $this->parameterBagColumns = array('P_CDFILIAL', 'P_NRTIPOCUSTO', 'P_TIPODATA', 'P_NRTIPOCUSTOREAL', 'P_DTINICIAL', 'P_DTMESINIC', 'P_DTFINAL', 'P_DTMESFINA', 'P_NRCONSPLAN', 'P_CDPRODINIC', 'P_CDPRODFINA', 'P_IDESTOCAVEIS', 'P_IDNESTOCAVEIS', 'P_DIFERENCA', 'P_SERVICOS', 'P_GRUPSERV', 'P_TIPO_REL');

        $filterCriteria = new FilterCriteria($this->dataSourceName);

        $filterCriteria->addCondition('P_CDFILIAL',     $row['CDFILIAL']);
        $filterCriteria->addCondition('P_NRTIPOCUSTO',  $row['CDTIPOCUSTOPLAN']);
        $filterCriteria->addCondition('P_TIPODATA',     $row['TPDTCUS']);
        $filterCriteria->addCondition('P_NRTIPOCUSTOREAL', $row['CDTIPOCUSTOREAL']);
        $filterCriteria->addCondition('P_DTINICIAL',    $row['DTINICIAL']);
        $filterCriteria->addCondition('P_DTMESINIC',    $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTFINAL',      $row['DTFINAL']);
        $filterCriteria->addCondition('P_DTMESFINA',    $row['DTMESFINA']);
        $filterCriteria->addCondition('P_NRCONSPLAN',   $row['NRCONSPLAN']);
        $filterCriteria->addCondition('P_CDPRODINIC',   $row['CDPRODINICIO']);
        $filterCriteria->addCondition('P_CDPRODFINA',   $row['CDPRODFINAL']);
        $filterCriteria->addCondition('P_IDESTOCAVEIS', $row['PRODESTOC']);
        $filterCriteria->addCondition('P_IDNESTOCAVEIS', $row['PRODNAOESTOC']);
        $filterCriteria->addCondition('P_DIFERENCA',    $row['IDDIFERENCA']);
        $filterCriteria->addCondition('P_SERVICOS',     $row['LISTASERV']);
        $filterCriteria->addCondition('P_GRUPSERV',     $row['LISTAGRUPSERV']);
        $filterCriteria->addCondition('P_TIPO_REL',     $row['TIPO_REL']);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }

    public function retornaServicosConcatenados($servicos) {
        $newArray = array();
        
        foreach ($servicos as $servico) {
            $newArray[] = $servico['CDSERVFILI'];
        }

        return implode("_", $newArray);
    }

    public function retornaArrayServfili($servicos) {
        $newArray = array();
        
        foreach ($servicos as $servico) {
            $newArray[] = $servico['CDSERVFILI'];
        }
        return $newArray;
    }

    // public function excluiConsplan(DTO\Request\Row $request, DTO\Response $response) {
    //     $parametros = array ("NRORG" => $this->environment->getNrOrg(), "CDFILICPLA" =>$this->environment->getCdFilial(), "NRCONSPLAN" => $this->codigo, "__is_new" => false);
    //     $newRequest = new DTO\Request\Row($parametros, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    //     $newResponse = new DTO\Response();
    //     $this->consplan->delete($newRequest, $newResponse);
    // }
}